@@ -1,6 +1,6 @@
 /*
 **  GNU Pth - The GNU Portable Threads
-**  Copyright (c) 1999-2000 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2002 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
 **  library which can be found at http://www.gnu.org/software/pth/.
@@ -29,10 +29,10 @@
 
     /* the library version */
 #ifndef PTH_VERSION_STR
-#define PTH_VERSION_STR "1.3.7 (29-Jul-2000)"
+#define PTH_VERSION_STR "1.4.1 (27-Jan-2002)"
 #endif
 #ifndef PTH_VERSION_HEX
-#define PTH_VERSION_HEX 0x103207
+#define PTH_VERSION_HEX 0x104201
 #endif
 #ifndef PTH_VERSION
 #define PTH_VERSION PTH_VERSION_HEX
@@ -215,7 +215,8 @@ struct pth_ringnode_st {
 };
 typedef struct pth_ring_st pth_ring_t;
 struct pth_ring_st {
-    pth_ringnode_t *rn_hook;
+    pth_ringnode_t *r_hook;
+    unsigned int    r_nodes;
 };
 #define PTH_RING_INIT { NULL }
 
@@ -243,12 +244,13 @@ enum { PTH_RWLOCK_RD, PTH_RWLOCK_RW };
 #define PTH_COND_SIGNALED            _BIT(1)
 #define PTH_COND_BROADCAST           _BIT(2)
 #define PTH_COND_HANDLED             _BIT(3)
-#define PTH_COND_INIT                { PTH_COND_INITIALIZED }
+#define PTH_COND_INIT                { PTH_COND_INITIALIZED, 0 }
 
    /* barrier variable values */
 #define PTH_BARRIER_INITIALIZED      _BIT(0)
-#define PTH_BARRIER_INIT(threshold)  { PTH_BARRIER_INITIALIZED, (threshold), 0, 0, \
-                                       PTH_MUTEX_INIT, PTH_COND_INIT }
+#define PTH_BARRIER_INIT(threshold)  { PTH_BARRIER_INITIALIZED, \
+                                       (threshold), (threshold), FALSE, \
+                                       PTH_COND_INIT, PTH_MUTEX_INIT }
 #define PTH_BARRIER_HEADLIGHT        (-1)
 #define PTH_BARRIER_TAILLIGHT        (-2)
 
@@ -364,10 +366,10 @@ struct iovec {
 #endif /* !_PTHREAD_PRIVATE */
 
     /* extension support */
-#define PTH_EXT_SFIO FALSE
+#define PTH_EXT_SFIO 0
 
     /* Sfio extension support */
-#if FALSE
+#if 0
 #include <sfio.h>
 #else
 typedef void *Sfdisc_t;
@@ -471,11 +473,16 @@ extern ssize_t        pth_read_ev(int, void *, size_t, pth_event_t);
 extern ssize_t        pth_write_ev(int, const void *, size_t, pth_event_t);
 extern ssize_t        pth_readv_ev(int, const struct iovec *, int, pth_event_t);
 extern ssize_t        pth_writev_ev(int, const struct iovec *, int, pth_event_t);
+extern ssize_t        pth_recv_ev(int, void *, size_t, int, pth_event_t);
+extern ssize_t        pth_send_ev(int, const void *, size_t, int, pth_event_t);
+extern ssize_t        pth_recvfrom_ev(int, void *, size_t, int, struct sockaddr *, socklen_t *, pth_event_t);
+extern ssize_t        pth_sendto_ev(int, const void *, size_t, int, const struct sockaddr *, socklen_t, pth_event_t);
 
     /* standard replacement functions */
 extern int            pth_usleep(unsigned int);
 extern unsigned int   pth_sleep(unsigned int);
 extern pid_t          pth_waitpid(pid_t, int *, int);
+extern int            pth_system(const char *);
 extern int            pth_sigmask(int, const sigset_t *, sigset_t *);
 extern int            pth_sigwait(const sigset_t *, int *);
 extern int            pth_connect(int, const struct sockaddr *, socklen_t);
@@ -486,6 +493,10 @@ extern ssize_t        pth_read(int, void *, size_t);
 extern ssize_t        pth_write(int, const void *, size_t);
 extern ssize_t        pth_readv(int, const struct iovec *, int);
 extern ssize_t        pth_writev(int, const struct iovec *, int);
+extern ssize_t        pth_recv(int, void *, size_t, int);
+extern ssize_t        pth_send(int, const void *, size_t, int);
+extern ssize_t        pth_recvfrom(int, void *, size_t, int, struct sockaddr *, socklen_t *);
+extern ssize_t        pth_sendto(int, const void *, size_t, int, const struct sockaddr *, socklen_t);
 extern ssize_t        pth_pread(int, void *, size_t, off_t);
 extern ssize_t        pth_pwrite(int, const void *, size_t, off_t);
 
@@ -494,10 +505,11 @@ END_DECLARATION
     /* soft system call mapping support */
 #if PTH_SYSCALL_SOFT && !defined(_PTH_PRIVATE)
 #define fork          pth_fork
+#define waitpid       pth_waitpid
+#define system        pth_system
 #define sleep         pth_sleep
 #define sigprocmask   pth_sigmask
 #define sigwait       pth_sigwait
-#define waitpid       pth_waitpid
 #define select        pth_select
 #define poll          pth_poll
 #define connect       pth_connect
@@ -506,6 +518,10 @@ END_DECLARATION
 #define write         pth_write
 #define readv         pth_readv
 #define writev        pth_writev
+#define recv          pth_recv
+#define send          pth_send
+#define recvfrom      pth_recvfrom
+#define sendto        pth_sendto
 #define pread         pth_pread
 #define pwrite        pth_pwrite
 #endif
